#coding: utf-8

import json
import time


class APIException(Exception):
    pass


def _prepare_time_spec(spec):
    assert ' ' in spec, 'Incorrect relative time specification!'
    (value, unit) = spec.split(' ')
    
    assert unit in ['milliseconds', 'seconds', 'minutes', 'hours', 
                    'days', 'weeks', 'months', 'years'], 'Incorrect time unit specified!'
    return {
        'unit': unit,
        'value': value
    }


class MetricsQuery(object):
    def __init__(self):
        self._metrics = []
        # Нижняя граница запроса данных - 1 января 1970 года.
        self._start_from = -10800
        self._end_to = None
        super(MetricsQuery, self).__init__()

    def add_metric(self, name, tags=None, aggregators=None):
        metric_rec = {
            'tags': tags or {},
            'name': name,
        }
        if aggregators:
            metric_rec['aggregators'] = [
                {
                    'name': n, 
                    'sampling': _prepare_time_spec(s)
                } for (n, s) in aggregators]

        self._metrics.append(metric_rec)

    def remove_metric(self, name):
        to_be_deleted = [
            m for m in self._metrics
            if m['name'] == name
        ]
        for m in to_be_deleted:
            self._metrics.remove(m)

    def set_start_from(self, value):
        if isinstance(value, str):
            self._start_from = _prepare_time_spec(value)
        elif value > -10800:
            self._start_from = value

    def set_end_to(self, value):
        if isinstance(value, str):
            self._end_to = _prepare_time_spec(value)
        elif value > -10800:
            self._end_to = value

    def __unicode__(self):
        request_data = {
            'metrics': self._metrics
        }
        start_key = 'start_relative' if isinstance(self._start_from, dict) else 'start_absolute'
        end_key = 'end_relative' if isinstance(self._end_to, dict) else 'end_absolute'
        request_data[start_key] = self._start_from
        request_data[end_key] = self._end_to or int(time.time() * 1000)

        return json.dumps(request_data, indent=4)
